/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tldgenerator;

import com.squeakysand.commons.javadoc.JavaDocHelper;
import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.jsp.tagext.annotations.JspElFunction;
import com.squeakysand.schema.javaee.DescriptionType;
import com.squeakysand.schema.javaee.FullyQualifiedClassType;
import com.squeakysand.schema.javaee.FunctionType;
import com.squeakysand.schema.javaee.ObjectFactory;
import com.squeakysand.schema.javaee.TldCanonicalNameType;

import com.sun.javadoc.AnnotationDesc;
import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.MethodDoc;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FunctionTypeListBuilder {

    private static final Logger LOG = LoggerFactory.getLogger(TldAttributeTypeListBuilder.class);

    private ClassDoc classDoc;
    private ObjectFactory objectFactory;

    public FunctionTypeListBuilder(ClassDoc classDoc) {
        this.classDoc = classDoc;
        objectFactory = new ObjectFactory();
    }

    public List<FunctionType> build() throws TldGeneratorException {
        LOG.debug("processing {} for functions", classDoc.qualifiedTypeName());
        List<FunctionType> result = new ArrayList<FunctionType>();
        MethodDoc[] methodDocs = classDoc.methods();
        for (MethodDoc methodDoc : methodDocs) {
            LOG.debug("processing method: {}", methodDoc.name());
            AnnotationDesc jspElFunctionAnnotation = JavaDocHelper.getAnnotation(methodDoc, JspElFunction.class.getSimpleName());
            if (jspElFunctionAnnotation != null) {
                LOG.debug("method has annotation");
                boolean isStatic = methodDoc.isStatic();
                boolean isPublic = methodDoc.isPublic();
                if (isStatic && isPublic) {
                    FunctionType function = objectFactory.createFunctionType();
                    // set the funciton class
                    FullyQualifiedClassType functionClass = objectFactory.createFullyQualifiedClassType();
                    functionClass.setValue(classDoc.qualifiedTypeName());
                    function.setFunctionClass(functionClass);
                    // set the function name
                    String name = (String) JavaDocHelper.getAnnotationElementValue(jspElFunctionAnnotation, "name");
                    if (StringUtils.isNullOrEmpty(name)) {
                        name = methodDoc.name();
                    }
                    TldCanonicalNameType xmlName = objectFactory.createTldCanonicalNameType();
                    xmlName.setValue(name);
                    function.setName(xmlName);
                    // set the signature
                    String methodSignature = JavaDocHelper.getMethodSignature(methodDoc);
                    com.squeakysand.schema.javaee.String xmlString = objectFactory.createString();
                    xmlString.setValue(methodSignature);
                    function.setFunctionSignature(xmlString);
                     // set the description
                    String functionDescription = methodDoc.commentText();
                    if (StringUtils.isNotNullOrEmpty(functionDescription)) {
                        List<DescriptionType> descriptions = function.getDescriptions();
                        DescriptionType xmlDescription = objectFactory.createDescriptionType();
                        xmlDescription.setValue(functionDescription);
                        descriptions.add(xmlDescription);
                    }
                    result.add(function);
                } else {
                    throw new TldGeneratorException(String.format("method %s with JspElFunction annotation is not public and static", methodDoc.name()));
                }
            } else {
                LOG.debug("method does not have annotaiton");
            }
        }
        return result;
    }

}
