/*
 * Copyright 2010-2011 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tldgenerator;

import com.squeakysand.commons.io.FileHelper;
import com.squeakysand.schema.javaee.ObjectFactory;
import com.squeakysand.schema.javaee.PathType;
import com.squeakysand.schema.javaee.TagFileType;
import com.squeakysand.schema.javaee.TldCanonicalNameType;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class TagFileTypeListBuilder {

    private ObjectFactory objectFactory;
    private File tagFilesDir;

    public TagFileTypeListBuilder(File tagFilesDir) {
        this.tagFilesDir = tagFilesDir;
        objectFactory = new ObjectFactory();
    }

    public List<TagFileType> build() throws TldGeneratorException {
        List<TagFileType> result = new ArrayList<TagFileType>();
        List<File> files = new ArrayList<File>();
        try {
            List<File> tagFiles = FileHelper.findFilesByExtension(tagFilesDir, "tag");
            files.addAll(tagFiles);
            List<File> tagFileDocuments = FileHelper.findFilesByExtension(tagFilesDir, "tagx");
            files.addAll(tagFileDocuments);
        } catch (IOException e) {
            throw new TldGeneratorException(e);
        }
        for (File file : files) {
            TagFileType tagFile = objectFactory.createTagFileType();
            // set the name
            String name = generateDefaultTagName(file);
            TldCanonicalNameType xmlName = objectFactory.createTldCanonicalNameType();
            xmlName.setValue(name);
            tagFile.setName(xmlName);
            // set the path
            String path = generateTagFilePath(file);
            PathType xmlPath = objectFactory.createPathType();
            xmlPath.setValue(path);
            tagFile.setPath(xmlPath);
            // add the tag to the list of existing tags
            result.add(tagFile);
        }
        return result;
    }

    private String generateDefaultTagName(File file) {
        String name = file.getName();
        // lowercase the first letter
        name = Character.toLowerCase(name.charAt(0)) + name.substring(1);
        // trim off the .tag file extension
        if (name.endsWith(".tag")) {
            name = name.substring(0, name.length() - 4);
        } else {
            // tag file name ends with .tagx instead
            name = name.substring(0, name.length() - 5);
        }
        return name;
    }

    private String generateTagFilePath(File file) {
        String fullFilePath = file.getPath();
        int index = fullFilePath.indexOf("/META-INF/tags");
        return fullFilePath.substring(index);
    }

}
